/*
 * Copyright (c) 2001 Mark Fullmer and The Ohio State University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      $Id: flow-cat.c,v 1.16 2001/03/14 03:27:04 maf Exp $
 */

#if HAVE_CONFIG_H
 #include <config.h>
#endif

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/signal.h>
#include <sys/uio.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#if HAVE_STRINGS_H
 #include <strings.h>
#endif
#if HAVE_STRING_H
  #include <string.h>
#endif
#include <time.h>
#include <fcntl.h>
#include "ftlib.h"
#include "support.h"

/* XXX flow-cat could be smart and translate among v1,5,6,7 export formats */

int debug;
int done;

void usage();
void sig_quit(int sig);

int main(int argc, char **argv)
{
  extern char *optarg;
  extern int optind, debug;
  int i, out_fd, out_fd_plain, in_fd, disable_mmap, in_fd_plain;
  char *fname, *out_fname;
  struct ftio ftio_in, ftio_out;
  struct ftprof ftp;
  struct ftver ftv, ftv2;
  struct ftset ftset;
  struct stat stat_buf;
  char *rec;
  u_int32 total_flows;
  u_int32 time_start, time_end, time_tmp;

  /* init fterr */
  fterr_setid(argv[0]);

  /* profile */
  ftprof_start (&ftp);

  bzero(&ftv, sizeof ftv);
  bzero(&ftv2, sizeof ftv2);

  /* defaults + no compression */
  ftset_init(&ftset, 0);

  total_flows = 0;
  out_fd_plain = 0;
  out_fname = (char*)0L;
  out_fd = -1;
  time_start = -1; /* MAXINT */
  time_end = 0;
  disable_mmap = 0;

  while ((i = getopt(argc, argv, "b:C:d:h?mo:z:")) != -1)

    switch (i) {

    case 'b': /* output byte order */
      if (!strcasecmp(optarg, "little"))
        ftset.byte_order = FT_HEADER_LITTLE_ENDIAN;
      else if (!strcasecmp(optarg, "big"))
        ftset.byte_order = FT_HEADER_BIG_ENDIAN;
      else 
        fterr_errx(1, "expecting \"big\" or \"little\");
      break;

    case 'C': /* comment field */
      ftset.comments = optarg;
      break;

    case 'd': /* debug */
      debug = atoi(optarg);
      break;

    case 'm': /* disable mmap */
      disable_mmap = 1;
      break;

    case 'o': /* output filename */
      out_fname = optarg;
      break;

    case 'z': /* compress level */
      ftset.z_level = atoi(optarg);
      if ((ftset.z_level < 0) || (ftset.z_level > 9))
        fterr_errx(1, Compression level must be between 0 and 9");
      break;

    case 'h': /* help */
    case '?': 
    default:
      usage();
      exit (1);
      break;

    } /* switch */

  i = optind;

  /* handle SIGQUIT */
  signal(SIGQUIT, sig_quit);
  signal(SIGHUP, sig_quit);
  signal(SIGINT, sig_quit);
  signal(SIGTERM, sig_quit);

  /* if out_fname is not set, then use stdout */
  if (out_fname) {

    if ((out_fd = open(out_fname,  O_WRONLY|O_CREAT|O_TRUNC, 0644)) == -1)
      fterr_err(1, "open(%s)", out_fname);

    if (fstat(out_fd, &stat_buf) == -1)
      fterr_err(1, "fstat(%s)", out_fname);

    /* is this a plain file? */
    if (!stat_buf.st_rdev)
      out_fd_plain = 1;

  } else 
    out_fd = 1;

  /* output to out_fd */    
  if (ftio_init(&ftio_out, out_fd, FT_IO_FLAG_WRITE |
    ((ftset.z_level) ? FT_IO_FLAG_ZINIT : 0) ) < 0)
    fterr_errx(1, "ftio_init(): failed");

  ftio_set_comment(&ftio_out, ftset.comments);
  ftio_set_byte_order(&ftio_out, ftset.byte_order);
  ftio_set_z_level(&ftio_out, ftset.z_level);
  ftio_set_streaming(&ftio_out, 1);
  ftio_set_debug(&ftio_out, debug);

  /* header must be full size on initial write */
  if (out_fd_plain) {
    ftio_set_cap_time(&ftio_out, time_start, time_end);
    ftio_set_flows_count(&ftio_out, total_flows);
  }

  /* handle special case of only stdin */
  if (i >= argc) {
    fname = "-";
    goto skip1;
  }

  for (; i < argc; ++i) {

    fname = argv[i];
skip1:

    if (debug > 5)
      fterr_info("fname=%s\n", fname);

    if (fname[0] == '-' && !fname[1]) {

      /* special case, stdin */
      in_fd = 0;

      in_fd_plain = 0;

    } else {

      if ((in_fd = open(fname, O_RDONLY, 0)) == -1) {
        fterr_warn("open(%s)", fname);
        continue;
      }

      in_fd_plain = 1;

    }

    if (ftio_init(&ftio_in, in_fd, FT_IO_FLAG_READ |
      ((in_fd_plain && !disable_mmap) ? FT_IO_FLAG_MMAP : 0)) < 0)
      fterr_errx(1, "ftio_init(): failed");

    ftio_get_ver(&ftio_in, &ftv2);

    time_tmp = ftio_get_cap_start(&ftio_in);
    if (time_tmp < time_start)
      time_start = time_tmp;

    time_tmp = ftio_get_cap_end(&ftio_in);
    if (time_tmp > time_end)
      time_end = time_tmp;

    /* first time? */
    if (!ftv.d_version) {

      /*
       * is this really the right thing to do here.  Reading a v1
       * stream gets handled by ftio_read(), but ftio_* leaves the
       * s_version at 1.
       */
      ftv2.s_version = FT_IO_SVERSION;

      /* set the version information in the io stream */
      if (ftio_set_ver(&ftio_out, &ftv2) < 0)
        fterr_errx(1, "ftio_set_ver(): failed");

      /* save for later compare */
      bcopy(&ftv2, &ftv, sizeof ftv);

      /* header first */
      if (ftio_write_header(&ftio_out) < 0)
        fterr_errx(1, "ftio_write_header(): failed");

    } else {

      /* ensure previous version == current version */
      if ((ftv.d_version != ftv2.d_version) ||
          (ftv.agg_method != ftv2.agg_method))
        fterr_errx(1, "data version or sub version changed!");

    }

    while ((rec = ftio_read(&ftio_in))) {

      ++total_flows;

      if (ftio_write(&ftio_out, rec) < 0)
        fterr_errx(1, "ftio_write(): failed");

      if (done)
        break;

    }

    if (ftio_close(&ftio_in) < 0)
      fterr_errx(1, "ftio_close(): failed");

    if (done)
      break;

  } /* foreach filename */


  /*
   * if the output file descriptor was a real file, re-write the
   * flow_header with the correct # of total flows
   */
  if (out_fd_plain) {

    ftio_set_cap_time(&ftio_out, time_start, time_end);
    ftio_set_flows_count(&ftio_out, total_flows);
    ftio_set_streaming(&ftio_out, 0);

    if (ftio_write_header(&ftio_out) < 0)
      fterr_errx(1, "ftio_write_header(): failed");

  } /* out_fd_plain */

  if (ftio_close(&ftio_out) < 0)
    fterr_errx(1, "ftio_close(): failed");

  if (debug > 0) {
    ftprof_end (&ftp, total_flows);
    ftprof_print(&ftp, argv[0], stderr);
  }

  return 0;
    

} /* main */

void usage() {

  fprintf(stderr, "flow-cat:\n\n");
  fprintf(stderr, " -b s  set output byte order (little or big)\n");
  fprintf(stderr, " -C s  set comment field in output\n");
  fprintf(stderr, " -d #  set debug level.\n");
  fprintf(stderr, " -m    disable use of mmap.\n");
  fprintf(stderr, " -o s  output to file s instead of stdout\n");
  fprintf(stderr, " -z #  set compression level (0..9).\n");
  fprintf(stderr, "\n\n");
} /* usage */

void sig_quit(int sig)
{
  extern int done;
  done = 1;
} /* sig_quit */

